@@ -18,6 +18,10 @@
 #include <string.h>
 #include <stdio.h>
 
+#ifdef _MSC_VER  /* This is only for Microsoft's compiler, not cygwin, e.g. */
+#define snprintf sprintf_s
+#endif
+
 static t_symbol *class_loadsym;     /* name under which an extern is invoked */
 static void pd_defaultfloat(t_pd *x, t_float f);
 static void pd_defaultlist(t_pd *x, t_symbol *s, int argc, t_atom *argv);
@@ -149,7 +153,6 @@ static void pd_defaultlist(t_pd *x, t_symbol *s, int argc, t_atom *argv)
     argument form, one for the multiple one; see select_setup() to find out
     how this is handled.  */
 
-extern t_widgetbehavior text_widgetbehavior;
 extern void text_save(t_gobj *z, t_binbuf *b);
 
 t_class *class_new(t_symbol *s, t_newmethod newmethod, t_method freemethod,
@@ -168,7 +171,7 @@ t_class *class_new(t_symbol *s, t_newmethod newmethod, t_method freemethod,
     {
         if (count == MAXPDARG)
         {
-            error("class %s: sorry: only %d creation args allowed",
+            error("class %s: sorry: only %d args typechecked; use A_GIMME",
                 s->s_name, MAXPDARG);
             break;
         }
@@ -300,6 +303,19 @@ void class_addmethod(t_class *c, t_method fn, t_symbol *sel,
     }
     else
     {
+        int i;
+        for (i = 0; i < c->c_nmethod; i++)
+            if (c->c_methods[i].me_name == sel)
+        {
+            char nbuf[80];
+            snprintf(nbuf, 80, "%s_aliased", sel->s_name);
+            c->c_methods[i].me_name = gensym(nbuf);
+            if (c == pd_objectmaker)
+                post("warning: class '%s' overwritten; old one renamed '%s'",
+                    sel->s_name, nbuf);
+            else post("warning: old method '%s' for class '%s' renamed '%s'",
+                sel->s_name, c->c_name->s_name, nbuf);
+        }
         c->c_methods = t_resizebytes(c->c_methods,
             c->c_nmethod * sizeof(*c->c_methods),
             (c->c_nmethod + 1) * sizeof(*c->c_methods));
@@ -400,7 +416,7 @@ static void pd_floatforsignal(t_pd *x, t_float f)
 {
     int offset = (*x)->c_floatsignalin;
     if (offset > 0)
-        *(t_sample *)(((char *)x) + offset) = f;
+        *(t_float *)(((char *)x) + offset) = f;
     else
         pd_error(x, "%s: float unexpected for signal input",
             (*x)->c_name->s_name);
@@ -459,12 +475,12 @@ t_propertiesfn class_getpropertiesfn(t_class *c)
 
 static t_symbol *symhash[HASHSIZE];
 
-t_symbol *dogensym(char *s, t_symbol *oldsym)
+t_symbol *dogensym(const char *s, t_symbol *oldsym)
 {
     t_symbol **sym1, *sym2;
     unsigned int hash1 = 0,  hash2 = 0;
     int length = 0;
-    char *s2 = s;
+    const char *s2 = s;
     while (*s2)
     {
         hash1 += *s2;
@@ -491,7 +507,7 @@ t_symbol *dogensym(char *s, t_symbol *oldsym)
     return (sym2);
 }
 
-t_symbol *gensym(char *s)
+t_symbol *gensym(const char *s)
 {
     return(dogensym(s, 0));
 }
@@ -753,19 +769,22 @@ badarg:
         s->s_name, c->c_name->s_name);
 }
 
+    /* convenience routine giving a stdarg interface to typedmess().  Only
+    ten args supported; it seems unlikely anyone will need more since
+    longer messages are likely to be programmatically generated anyway. */
 void pd_vmess(t_pd *x, t_symbol *sel, char *fmt, ...)
 {
     va_list ap;
-    t_atom arg[MAXPDARG], *at =arg;
+    t_atom arg[10], *at = arg;
     int nargs = 0;
     char *fp = fmt;
 
     va_start(ap, fmt);
     while (1)
     {
-        if (nargs > MAXPDARG)
+        if (nargs >= 10)
         {
-            pd_error(x, "pd_vmess: only %d allowed", MAXPDARG);
+            pd_error(x, "pd_vmess: only 10 allowed");
             break;
         }
         switch(*fp++)
