@@ -35,6 +35,7 @@ void sys_findprogdir(char *progname);
 int sys_startgui(const char *guipath);
 int sys_rcfile(void);
 int m_mainloop(void);
+int m_batchmain(void);
 void sys_addhelppath(char *p);
 #ifdef USEAPI_ALSA
 void alsa_adddev(char *name);
@@ -50,7 +51,7 @@ int sys_nosleep = 0;  /* skip all "sleep" calls and spin instead */
 
 char *sys_guicmd;
 t_symbol *sys_libdir;
-static t_symbol *sys_guidir;
+t_symbol *sys_guidir;
 static t_namelist *sys_openlist;
 static t_namelist *sys_messagelist;
 static int sys_version;
@@ -75,10 +76,12 @@ static int sys_listplease;
 
 int sys_externalschedlib;
 char sys_externalschedlibname[MAXPDSTRING];
+static int sys_batch;
 int sys_extraflags;
 char sys_extraflagsstring[MAXPDSTRING];
 int sys_run_scheduler(const char *externalschedlibname,
     const char *sys_extraflagsstring);
+int sys_noautopatch;    /* temporary hack to defeat new 0.42 editing */
 
     /* here the "-1" counts signify that the corresponding vector hasn't been
     specified in command line arguments; sys_set_audio_settings will detect it
@@ -282,7 +285,8 @@ int sys_main(int argc, char **argv)
     if (!noprefs)
         sys_loadpreferences();                  /* load default settings */
 #ifndef MSW
-    sys_rcfile();                               /* parse the startup file */
+    if (!noprefs)
+        sys_rcfile();                           /* parse the startup file */
 #endif
     if (sys_argparse(argc-1, argv+1))           /* parse cmd line */
         return (1);
@@ -298,6 +302,8 @@ int sys_main(int argc, char **argv)
     if (sys_externalschedlib)
         return (sys_run_scheduler(sys_externalschedlibname,
             sys_extraflagsstring));
+    else if (sys_batch)
+        return (m_batchmain());
     else
     {
             /* open audio and MIDI */
@@ -328,7 +334,6 @@ static char *(usagemessage[]) = {
 
 #ifdef USEAPI_OSS
 "-oss             -- use OSS audio API\n",
-"-32bit           ----- allow 32 bit OSS audio (for RME Hammerfall)\n",
 #endif
 
 #ifdef USEAPI_ALSA
@@ -391,6 +396,10 @@ static char *(usagemessage[]) = {
 "-nrt             -- don't use real-time priority\n",
 #endif
 "-nosleep         -- spin, don't sleep (may lower latency on multi-CPUs)\n",
+"-schedlib <file> -- plug in external scheduler\n",
+"-extraflags <s>  -- string argument to send schedlib\n",
+"-batch           -- run off-line as a batch process\n",
+"-noautopatch     -- defeat auto-patching new from selected objects\n",
 };
 
 static void sys_parsedevlist(int *np, int *vecp, int max, char *str)
@@ -611,12 +620,6 @@ int sys_argparse(int argc, char **argv)
             sys_set_audio_api(API_OSS);
             argc--; argv++;
         }
-        else if (!strcmp(*argv, "-32bit"))
-        {
-            sys_set_audio_api(API_OSS);
-            oss_set32bit();
-            argc--; argv++;
-        }
 #endif
 #ifdef USEAPI_ALSA
         else if (!strcmp(*argv, "-alsa"))
@@ -809,7 +812,7 @@ int sys_argparse(int argc, char **argv)
             sys_listplease = 1;
             argc--; argv++;
         }
-        else if (!strcmp(*argv, "-schedlib"))
+        else if (!strcmp(*argv, "-schedlib") && argc > 1)
         {
             sys_externalschedlib = 1;
             strncpy(sys_externalschedlibname, argv[1],
@@ -817,7 +820,7 @@ int sys_argparse(int argc, char **argv)
             argv += 2;
             argc -= 2;
         }
-        else if (!strcmp(*argv, "-extraflags"))
+        else if (!strcmp(*argv, "-extraflags") && argc > 1)
         {
             sys_extraflags = 1;
             strncpy(sys_extraflagsstring, argv[1],
@@ -825,6 +828,17 @@ int sys_argparse(int argc, char **argv)
             argv += 2;
             argc -= 2;
         }
+        else if (!strcmp(*argv, "-batch"))
+        {
+            sys_batch = 1;
+            sys_printtostderr = sys_nogui = 1;
+            argc--; argv++;
+        }
+        else if (!strcmp(*argv, "-noautopatch"))
+        {
+            sys_noautopatch = 1;
+            argc--; argv++;
+        }
 #ifdef UNISTD
         else if (!strcmp(*argv, "-rt") || !strcmp(*argv, "-realtime"))
         {
