@@ -69,6 +69,24 @@ void sys_unbashfilename(const char *from, char *to)
     *to = 0;
 }
 
+/* test if path is absolute or relative, based on leading /, env vars, ~, etc */
+int sys_isabsolutepath(const char *dir)
+{
+    if (dir[0] == '/' || dir[0] == '~'
+#ifdef MSW
+        || dir[0] == '%' || (dir[1] == ':' && dir[2] == '/')
+#endif
+        )
+    {
+        return 1;
+    }
+    else
+    {
+        return 0;            
+    }
+}
+
+
 /*******************  Utility functions used below ******************/
 
 /*!
@@ -250,14 +268,13 @@ int sys_trytoopenone(const char *dir, const char *name, const char* ext,
 int sys_open_absolute(const char *name, const char* ext,
     char *dirresult, char **nameresult, unsigned int size, int bin, int *fdp)
 {
-    if (name[0] == '/' 
-#ifdef MSW
-        || (name[1] == ':' && name[2] == '/')
-#endif
-            )
+    if (sys_isabsolutepath(name))
     {
-        char dirbuf[MAXPDSTRING];
-        int dirlen = (strrchr(name, '/') - name);
+        char dirbuf[MAXPDSTRING], *z = strrchr(name, '/');
+        int dirlen;
+        if (!z)
+            return (0);
+        dirlen = z - name;
         if (dirlen > MAXPDSTRING-1) 
             dirlen = MAXPDSTRING-1;
         strncpy(dirbuf, name, dirlen);
@@ -405,9 +422,9 @@ int sys_rcfile(void)
 
     for (i = 1; i < NUMARGS-1; i++)
     {
-        if (fscanf(file, "%999s", buf) < 0)
+        if (fscanf(file, "%998s", buf) < 0)
             break;
-        buf[1000] = 0;
+        buf[999] = 0;
         if (!(rcargv[i] = malloc(strlen(buf) + 1)))
             goto cleanup;
         strcpy(rcargv[i], buf);
@@ -423,7 +440,7 @@ int sys_rcfile(void)
     fclose(file);
     if (sys_verbose)
     {
-        if (rcargv)
+        if (rcargc)
         {
             post("startup args from RC file:");
             for (i = 1; i < rcargc; i++)
